﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Windows;
using Infragistics.Controls.Grids;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using System.Windows.Data;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;

namespace PivotGridTestApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Dictionary<PivotCell, List<int>> SelectedRecords = new Dictionary<PivotCell, List<int>>();

        public MainWindow()
        {
            InitializeComponent();
            pivotGrid.Loaded += FlatDataLoaded;
        }

        private void FlatDataLoaded(object sender, RoutedEventArgs e)
        {
            var flatDataSource = new FlatDataSource
            {
                ItemsSource = CreateCollection(),
                Cube = DataSourceBase.GenerateInitialCube("PivotGridData"),
                Rows = DataSourceBase.GenerateInitialItems("[XAxis].[XAxis]"),
                Columns = DataSourceBase.GenerateInitialItems("[Product].[Product]"),
                Measures = DataSourceBase.GenerateInitialItems("Value")
            };
            pivotGrid.DataSource = flatDataSource;
            dataSelector.DataSource = flatDataSource;
        }

        private IEnumerable CreateCollection()
        {
            var products = new List<PivotGridData>();
            for (var i = 0; i < 100; i++)
            {
                PivotGridData product;
                if(i<10)
                    product = new PivotGridData(i.ToString(), "Product1", 123.12345);
                else if(i<20)
                    product = new PivotGridData(i.ToString(), "Product2", -123.12345);
                else if (i < 20)
                    product = new PivotGridData(i.ToString(), "Product3", 123.12345);
                else if (i < 30)
                    product = new PivotGridData(i.ToString(), "Product4", 123.12345);
                else if (i < 40)
                    product = new PivotGridData(i.ToString(), "Product5", -123.12345);
                else 
                    product = new PivotGridData(i.ToString(), "Product6", 123.12345);
                products.Add(product);
            }
            return products;
        }

        private void PivotGridCellControlAttached(object sender, PivotCellControlAttachedEventArgs e)
        {
            e.IsDirty = true;
            if (pivotGrid.DataSource == null) return;

            var rowIndex = pivotGrid.DataRows.IndexOf(e.Cell.DataRow);
            var columnIndex = pivotGrid.DataColumns.IndexOf(e.Cell.DataColumn);
            ICell cellData = pivotGrid.DataSource.Result.Cells[rowIndex, columnIndex];

            if (cellData == null) return;

            var value = 0.0;
            if (!double.TryParse(cellData.Value.ToString(), out value)) return;
            Style s = null;
            if (value < 0)
            s = Resources["RedStyle"] as Style;
            if (e.Cell.Style == null || e.Cell.Style != s)
                e.Cell.Style = s;
        }

        private void OnPivotCellLoaded(object sender, RoutedEventArgs e)
        {

        }

        private void CurvesDataGrid_Unloaded(object sender, RoutedEventArgs e)
        {
            
        }

        private void CurvesDataGrid_SelectedItemsChanged(object sender, Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs e)
        {
            XamDataGrid xdg = sender as XamDataGrid;
            PivotCellControl control = (PivotCellControl)Utilities.GetAncestorFromType(sender as XamDataGrid, typeof(PivotCellControl), false);
            if (control != null && control.Cell != null)
            {
                // Check to see if this datagrid is already being kept track of.  If not let's start tracking it
                if (!SelectedRecords.ContainsKey(control.Cell as PivotCell))
                    SelectedRecords.Add(control.Cell as PivotCell, new List<int>());
                
                // Keep track of what records were selected
                PivotCell cell = control.Cell as PivotCell;
                SelectedRecords[cell].Clear();
                foreach (var record in xdg.SelectedItems.Records)
                    SelectedRecords[cell].Add(record.Index);
            }
        }

        private void CurvesDataGrid_Loaded(object sender, RoutedEventArgs e)
        {
            XamDataGrid xdg = sender as XamDataGrid;
            PivotCellControl control = (PivotCellControl)Utilities.GetAncestorFromType(sender as XamDataGrid, typeof(PivotCellControl), false);
            if (control != null && control.Cell != null)
            {
                // a new datagrid instance has been created but we don't need to restore selected records 
                // if none were selected in the first place.
                if (SelectedRecords.ContainsKey(control.Cell as PivotCell))
                {
                    // select the records.
                    PivotCell cell = control.Cell as PivotCell;
                    foreach (int index in SelectedRecords[cell])
                    {
                        xdg.SelectedItemsChanged -= CurvesDataGrid_SelectedItemsChanged;
                        xdg.Records[index].IsSelected = true;
                        xdg.SelectedItemsChanged += CurvesDataGrid_SelectedItemsChanged;
                    }
                }
            }
        }
    }
    public class FormatValueConverter : IValueConverter
    {
        private string GetFormatString(string product)
        {
            switch (product)
            {
                case "Product1":
                    return "{0:0.0}";
                case "Product2":
                    return "{0:0.00}";
                case "Product3":
                    return "{0:0.000}";
                case "Product4":
                    return "{0:0.0000}";
                case "Product5":
                    return "{0:0.0%}";
                case "Product6":
                    return "{0:0.00%}";
                case "Product7":
                    return "{0:0.000%}";
                case "Product8":
                    return "{0:0.0000%}";
                case "Product9":
                    return "{0:0}";
                default:
                    return "{0:0}";
            }
        }
        #region IValueConverter Members


        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value != null)
            {
                PivotCell pCell = (value as PivotCell);
                ICell cell = pCell.Data as ICell;
                if (cell != null)
                {
                    string productFormat = GetFormatString(pCell.DataColumn.HeaderText);
                    return string.Format(productFormat, cell.Value);
                }
            }

            return "";
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return "";
        }

        #endregion
    }
 
}
